
import torch

def comp(A) :
    if A.dim() == 3 :
        a = A.detach().cpu().numpy()
        return a[:,:,0] + 1j*a[:,:,1]

def rot(x_i, θ) :
    "Rotate an N-by-2 array by angle θ."
    return torch.stack( ( torch.cos(θ)*x_i[:,0] - torch.sin(θ)*x_i[:,1],
                          torch.sin(θ)*x_i[:,0] + torch.cos(θ)*x_i[:,1]), 1 )

def normalize( x ) :
    d = 0 if x.dim()==2 else 1
    y = x - x.mean(d).unsqueeze(d)
    norms = (y**2).sum(-1).sum(-1).sqrt()
    norms = norms.view(-1,1) if x.dim()==2 else norms.view(-1,1,1)
    return y / (norms + 1e-5)

def herm( x, y ) :
    "Hermitian product"
    if x.dim()==2 and y.dim()==2 :
        r = (x*y).sum()
        i = (x[:,0]*y[:,1]-x[:,1]*y[:,0]).sum()
        return torch.stack( (r,i) )
    elif x.dim()==3 and y.dim()==2 :
        y = y.unsqueeze(0)
        r = (x*y).sum(2).sum(1)
        i = (x[:,:,0]*y[:,:,1]-x[:,:,1]*y[:,:,0]).sum(1)
        return torch.stack( (r,i), -1 )
    else :
        raise NotImplementedError()

def angle( x ) :
    return torch.atan2(x[1], x[0]) if x.dim()==1 else torch.atan2(x[:,1], x[:,0])

def mod2( x ) :
    return (x**2).sum(-1)

def mod( x ) :
    return mod2(x).sqrt()